import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A framework for other blocks. Has a life which decreases by one each time it's hit.
 * 
 * @author Michael Berry
 * @version 1.0
 */
public abstract class Block extends Actor
{
    /** The life of the block */
    private int life;
    /** The initial starting life of the block*/
    private int startLife;
    /** 1/this chance of a powerup appearing when the block is destroyed */
    public static final int POWERUPCHANCE = 10;
    /** The red block */
    public static final GreenfootImage red = new GreenfootImage("images/redBlock.png");
    /** The green block */
    public static final GreenfootImage green = new GreenfootImage("images/greenBlock.png");
    /** The blue block */
    public static final GreenfootImage blue = new GreenfootImage("images/blueBlock.png");
    
    /**
     * The act method - check to see whether the object has been hit by a ball
     */
    public void act()
    {
        if(getOneIntersectingObject(Ball.class)!=null) {
            hit();
            takeLife();
        }
    }
    
    /**
     * Gets the remaining life of the block
     * @return the remaining life
     */
    public int getLife()
    {
        return life;
    }
    
    /**
     * Create a new block with a specified starting life
     * @param startLife the starting life of the block
     */
    public Block(int startLife)
    {
        if(startLife<=0) throw new RuntimeException(startLife + ": Invalid starting life! Must be greater than 0");
        this.startLife = startLife;
        life = startLife;
    }
    
    /**
     * Called when the block has been hit. Subclasses need to over-ride this
     * if they want the block to do something specific when the ball hits it.
     * This is not abstract, just empty since most blocks will not require
     * anything special.
     */
    public void hit(){}
    
    /**
     * Takes a life from the block
     */
    public void takeLife()
    {
        life--;
        Board board = (Board)getWorld();
        board.addScore(10);
        if(life==0) {
            if(Greenfoot.getRandomNumber(POWERUPCHANCE)==0) createPowerup();
            board.removeObject(this);
            if(board.getObjects(Block.class).size()==0) {
                board.nextLevel();
            }
        }
    }
    
        
    /**
     * Choose a random powerup and creates it
     */
    public void createPowerup()
    {
        Board board = (Board)getWorld();
        int num = Greenfoot.getRandomNumber(Powerup.powerups.length);
        switch(num) {
            case 0:board.addObject(new SpeedPower(), getX(), getY());break;
            case 1:board.addObject(new BallPower(), getX(), getY());break;
            case 2:board.addObject(new LifePower(), getX(), getY());break;
            case 3:board.addObject(new Bonus(), getX(), getY());break;
            default:break;
        }
    }
    
    
}
